--[[ See license.txt for license and copyright information ]]
local _, addon = ...

local C_ContributionCollector = C_ContributionCollector
local C_DateAndTime = C_DateAndTime
local C_QuestLog = C_QuestLog
local C_TaskQuest = C_TaskQuest
local CreateFrame = CreateFrame
local UnitFactionGroup = UnitFactionGroup

local rulePrepend = "local g, o = ... "
local warfrontStates = { 1, 1, 2, 2 }
addon.ruleListeners = { }

local interval, minInterval, maxInterval = 1/15, 1/15, 10.0
local timeSinceLastUpdate = 0

local visibilityRules = {
	["default"] = function() return true end,
	["arathi"] = loadstring(rulePrepend .. "return g.arathiPhase and (o.Phase == 3 or o.Phase == g.arathiPhase) and (o.Hostile == 3 or o.Hostile == g.playerFaction)"),
	["darkshore"] = loadstring(rulePrepend .. "return g.darkshorePhase and (o.Phase == 3 or o.Phase == g.darkshorePhase) and (o.Hostile == 3 or o.Hostile == g.playerFaction)"),
	["uldum"] = loadstring(rulePrepend .. "return g.uldumPhase and bit.band(o.Phase, g.uldumPhase) == g.uldumPhase"),
	["vale"] = loadstring(rulePrepend .. "return g.valePhase and bit.band(o.Phase, g.valePhase) == g.valePhase"),
}

local locationRules = {
	["default"] = loadstring(rulePrepend .. "return o.Locations[1]"),
	["arathi"] = loadstring(rulePrepend .. "if #o.Locations == 1 then return o.Locations[1] end return o.Locations[g.arathiPhase]")
}

local trackingRules = {
	["default"] = loadstring(rulePrepend .. "return C_QuestLog.IsQuestFlaggedCompleted(o.VisibleTrackingQuestID)"),
	["arathi"] = loadstring(rulePrepend .. "return C_QuestLog.IsQuestFlaggedCompleted(o.PrevQuestID[(o.Hostile == 3 and g.playerFaction) or 1])"),
	["darkshore"] = loadstring(rulePrepend .. "return C_QuestLog.IsQuestFlaggedCompleted(o.VisibleTrackingQuestID) or C_QuestLog.IsQuestFlaggedCompleted(o.PrevQuestID[(o.Hostile == 3 and g.playerFaction) or 1])")
}

local function executeRule(ruleSet, ruleName, ...)
	return ruleSet[ruleName](addon.globals, ...)
end

function addon.executeVisibilityRule(ruleName, ...)
	return executeRule(visibilityRules, ruleName, ...)
end

function addon.executeLocationRule(ruleName, ...)
	return executeRule(locationRules, ruleName, ...)
end

function addon.executeTrackingRule(ruleName, ...)
	return executeRule(trackingRules, ruleName, ...)
end

local function NotifyRuleListeners()
	if (rawget(addon.globals,"changed")) then
		rawset(addon.globals,"changed", nil)
		for _, v in ipairs(addon.ruleListeners) do
			v()
		end
	end
end

local function OnUpdate(_, elapsed)
	timeSinceLastUpdate = timeSinceLastUpdate + elapsed
	if (timeSinceLastUpdate > interval) then
		timeSinceLastUpdate = 0
		addon.globals.arathiPhase = warfrontStates[C_ContributionCollector.GetState(11)]
		addon.globals.darkshorePhase = warfrontStates[C_ContributionCollector.GetState(118)]
		local secondsUntilWeeklyReset = C_DateAndTime.GetSecondsUntilWeeklyReset()
		if C_TaskQuest.GetQuestTimeLeftMinutes(58705) or C_QuestLog.IsQuestFlaggedCompleted(58705) then
			addon.globals.uldumPhase = 1
			if secondsUntilWeeklyReset < 302400 then
				addon.globals.valePhase = 4
			else
				addon.globals.valePhase = 2
			end
		elseif C_TaskQuest.GetQuestTimeLeftMinutes(55466) or C_QuestLog.IsQuestFlaggedCompleted(55466) then
			addon.globals.valePhase = 1
			if secondsUntilWeeklyReset < 302400 then
				addon.globals.uldumPhase = 4
			else
				addon.globals.uldumPhase = 2
			end
		else
			addon.globals.uldumPhase = nil
			addon.globals.valePhase = nil
		end
		if (addon.globals.arathiPhase and addon.globals.darkshorePhase) then
			interval = maxInterval
		else
			interval = minInterval
		end
	end
	NotifyRuleListeners()
end

local g = { }

g.factions = { Alliance = 1, Horde = 2, Neutral = 3 }
g.playerFaction = g.factions[UnitFactionGroup("player")]
g.playerIsAlliance = g.playerFaction == g.factions.Alliance
g.playerIsHorde = g.playerFaction == g.factions.Horde

addon.globals = { }

setmetatable(addon.globals, {
	__index = function(_,k)
		return g[k]
	end,
	__newindex = function(t,k,v)
		if (g[k] ~= v) then
			rawset(t,"changed", true)
			g[k] = v
		end
	end
})

local frame = CreateFrame("FRAME")
OnUpdate(frame, maxInterval + 1)
frame:SetScript("OnUpdate", OnUpdate)
